/***********************************************************************
 * Macro qui assigne les deux dimensions de l'indice de          *******
 * dfavorisation qubcois et des indices locaux (quartiles     *******
 * pour les RLS et les rgions administratives), la zone         *******
 *  partir du code postal et du code municipal  tout fichier   *******
 * administratif comprenant au moins le code postal.             *******
 * On utilise une version simplifie du FCCP+ produit par        *******
 * Statistique Canada. En tout, le programme fait appel          *******
 *  3 fichiers de conversion.                                   *******
 *  - WCF du FCCP+                                               *******
 *  - PCCFDUPS du FCCP+                                          *******
 *  - PCCFUNIQ du FCCP+                                          *******
 * Paramtres de la macro                                        *******
 *  - IN: fichier SAS d'entre auquel on dsire ajouter          *******
 *        l'indice de dfavorisation (materiel et social)        *******
 *  - PCODE: nom de la variable du code postal  6 positions     *******
 *  - MUNIC: nom de la variable du code municipal. Par dfaut,   *******
 *           0 = l'attribution ne se fait qu' partir du code    *******
 *           postal seulement                                    *******
 *  - OUT: nom du fichier SAS de sortie identique  IN mais avec *******
 *         les nouvelles variables AD et indices de              *******
 *         dfavorisation                                        *******
 * Auteur: Denis Hamel, INSPQ                                    *******
 *         Philippe Gamache, INSPQ                               *******
 * Dernire modification: 14 janvier 2020                        *******
 * Modifications: Ajout d'un libname pour transformer les        *******
 *                donnes de dpart en fichiers permanants       *******
 ***********************************************************************/

filename fcp "D:\Projets\Indice dfavorisation\Indice 2020\Assignation\Canada\fcp.txt";
filename pccfuniq "D:\Projets\Indice dfavorisation\Indice 2020\Assignation\Canada\fccpindiceuniq.txt";
filename pccfdups "D:\Projets\Indice dfavorisation\Indice 2020\Assignation\Canada\fccpindicedouble.txt";
filename munic "D:\Projets\Indice dfavorisation\Indice 2020\Assignation\Canada\munic.txt";
libname i2006 "D:\Projets\Indice dfavorisation\Indice 2020\Assignation\Canada";


data i2006.wcf;
infile fcp;
input  munic 1-12 pcode $ 13-18 propmunic 19-23 propcp 24-28 quintmat 29
       quintsoc 30 centmat 31-33 centsoc 34-36 diffmatcp 37 diffsoccp 38
       diffmatmunic 39 diffsocmunic 40 fortcp 41 fortmunic 42
       zone 44 zim 45 pr 46-47 quintmatRC 48 quintsocRC 49 quintmatRMR 50 quintsocRMR 51 
       quintmatZONE 52 quintsocZONE 53;
run;

data i2006.pccfuniq;
infile pccfuniq;
input  munic 1-12 pcode $ 13-18 propmunic 19-23 propcp 24-28 quintmat 29
       quintsoc 30 centmat 31-33 centsoc 34-36 diffmatcp 37 diffsoccp 38
       diffmatmunic 39 diffsocmunic 40 fortcp 41 fortmunic 42
       zone 44 zim 45 pr 46-47 quintmatRC 48 quintsocRC 49 quintmatRMR 50 quintsocRMR 51
       quintmatZONE 52 quintsocZONE 53;
run;

data i2006.pccfdups;
infile pccfdups;
input  munic 1-12 pcode $ 13-18 propmunic 19-23 propcp 24-28 quintmat 29
       quintsoc 30 centmat 31-33 centsoc 34-36 diffmatcp 37 diffsoccp 38
       diffmatmunic 39 diffsocmunic 40 fortcp 41 fortmunic 42
       zone 44 zim 45 pr 46-47 quintmatRC 48 quintsocRC 49 quintmatRMR 50 quintsocRMR 51
       quintmatZONE 52 quintsocZONE 53;
run;

data i2006.munic;
  infile munic;
  input munic 1-7 mun96 8-14 mun01 15-21;
run;

%macro presence(var=);
%global presence;
%let dsid=%sysfunc(open(fichin,i));
    %if &dsid>0 %then
       %do;
	     %let presence=%sysfunc(varnum(&dsid,&var));
		 /*** la macrovariable presence = 0 si la variable est absente du fichier ****/
	   %end;
%put &presence;
%let rc = %sysfunc(close(&dsid));
%mend presence;


/**** Ne pas changer la macro qui suit *****/
%macro assignation2006(in,pcode,munic,out);

%if %bquote(&munic)=%str() %then %let munic=0; ;

data wcf;
   set i2006.wcf;
   nuniq = _n_;
run;

data pccfuniq;
   set i2006.pccfuniq;
   nuniq = _n_;
run;

data pccfdups;
   set i2006.pccfdups;
   nuniq = _n_;
run;

  data fichin;
    %if &munic ne 0 %then %do; length munic 8.; %end;
    set &in;
    numero=_n_;
	nba=ranuni(766);
  /*** Mettre ici le nom de la variable de code postal 
	   et du code municipal ***/
	pcode=compress(&pcode);
	%if &munic ne 0 %then 
     %do; 
       if substr(pcode,1,1) in ("A") then munic=1000000+mod(&munic,1000000);
       else if substr(pcode,1,1) in ("B") then munic=1200000+mod(&munic,1200000);
       else if substr(pcode,1,1) in ("C") then munic=1100000+mod(&munic,1100000);
       else if substr(pcode,1,1) in ("E") then munic=1300000+mod(&munic,1300000);
       else if substr(pcode,1,1) in ("G","H","J") then munic=2400000+mod(&munic,2400000);
       else if substr(pcode,1,1) in ("K","L","M","N","P") then munic=3500000+mod(&munic,3500000);
       else if substr(pcode,1,1) in ("R") then munic=4600000+mod(&munic,4600000);
       else if substr(pcode,1,1) in ("S") then munic=4700000+mod(&munic,4700000);
       else if substr(pcode,1,1) in ("T") then munic=4800000+mod(&munic,4800000);
       else if substr(pcode,1,1) in ("V") then munic=5900000+mod(&munic,5900000);
       else if substr(pcode,1,1) in ("X") then munic=6100000+mod(&munic,6100000);
       else if substr(pcode,1,1) in ("Y") then munic=6000000+mod(&munic,6000000);
     %end;
   run;

%presence(var=quintmat);

%if &presence ne 0 %then %do;
   data fichin;
   set fichin;
   where zone in (.,0);
   run;
%end;

data fichin;
  set fichin;
  keep numero nba pcode %if &munic ne 0 %then %do; munic %end;;
run;

%if &munic ne 0 %then %do;

  proc sort data=i2006.munic;
    by munic;
   run;

  proc sort data=fichin;
    by munic;
   run;

  data fichin;
    merge i2006.munic(in=in2) fichin(in=in1);
	by munic;
	if in1*in2 then munic=mun01;
	else if in2 then delete;
   run;

/*********************************************************
  * Assignation des indices de dfavorisation  partir ***
  * du code postal et du code de la municipalit       ***
  * Fichier FCCP+                                      ***
  *******************************************************/

/*** Avec WCF *****/

   proc sql;
     create table test as
     select fichin.pcode, fichin.munic, numero, nba, wcf.propmunic, 
	        wcf.diffmatmunic, wcf.diffsocmunic, wcf.fortcp, wcf.nuniq,
            wcf.fortmunic, wcf.quintmat, wcf.quintsoc, wcf.centmat, wcf.centsoc, wcf.zim, wcf.zone,
			wcf.pr, wcf.quintmatRC, wcf.quintsocRC, wcf.quintmatRMR, wcf.quintsocRMR, wcf.quintmatZONE, wcf.quintsocZONE
     from fichin, wcf
     where fichin.pcode=wcf.pcode and fichin.munic=wcf.munic
	 order by numero,nuniq; 
    quit;

  proc sort data=test;
    by pcode munic numero;
   run;

   /*** Choix de l'indice de dfavorisation si plus d'un indice
        par code postal ***/
  data test;
    set test;
    by pcode munic numero;
	retain cumul find direct;
	if first.numero then do; find=0; cumul=0; 
                             if last.numero then direct=1; else direct=0;
                         end;
	if find = 0 then do;
	   cumul=cumul+(propmunic/100);
	   if nba < cumul then do; find=1; output; end;
      end;
   run; 

  proc sort data=test;
    by numero;
   run;

  proc sort data=fichin;
    by numero;
   run; 

  data appmunic1 nonapp1;
    merge fichin(in=in1) test(in=in2);
	by numero;
	if in1*in2 then output appmunic1;
	else if in1 then output nonapp1;
   run;

 /*** Avec PCCFUNIQ *****/

  proc sort data=nonapp1;
    by pcode munic;
   run;

  proc sort data=pccfuniq;
    by pcode munic;
   run;

   proc sql;
     create table appmunic2 as
     select nonapp1.pcode, nonapp1.munic, numero, nba, pccfuniq.propmunic, 
	        pccfuniq.diffmatmunic, pccfuniq.diffsocmunic, pccfuniq.fortcp,
            pccfuniq.fortmunic, pccfuniq.quintmat, pccfuniq.quintsoc, pccfuniq.centmat, pccfuniq.centsoc,
			pccfuniq.zim, pccfuniq.zone,pccfuniq.nuniq,
			pccfuniq.pr, pccfuniq.quintmatRC, pccfuniq.quintsocRC, pccfuniq.quintmatRMR, pccfuniq.quintsocRMR, 
            pccfuniq.quintmatZONE, pccfuniq.quintsocZONE
     from nonapp1, pccfuniq
     where nonapp1.pcode=pccfuniq.pcode and nonapp1.munic=pccfuniq.munic
	 order by numero,nuniq; 
    quit;

  data nonapp2;
    merge nonapp1(in=in2) pccfuniq(in=in1);
	by pcode munic;
	if in2 and not(in1);
   run;

 /*** Avec PCCFDUPS *****/

   proc sql;
     create table test as
     select nonapp2.pcode, nonapp2.munic, numero, nba, pccfdups.propmunic, 
	        pccfdups.diffmatmunic, pccfdups.diffsocmunic, pccfdups.fortcp,pccfdups.nuniq,
            pccfdups.fortmunic, pccfdups.quintmat, pccfdups.quintsoc, pccfdups.centmat, pccfdups.centsoc, pccfdups.zim, pccfdups.zone,
			pccfdups.pr, pccfdups.quintmatRC, pccfdups.quintsocRC, pccfdups.quintmatRMR, pccfdups.quintsocRMR,
            pccfdups.quintmatZONE, pccfdups.quintsocZONE
     from nonapp2, pccfdups
     where nonapp2.pcode=pccfdups.pcode and nonapp2.munic=pccfdups.munic
	 order by numero,nuniq; 
    quit;

  proc sort data=test;
    by pcode munic numero;
   run;

   /*** Choix de l'indice de dfavorisation si plus d'un indice
        par code postal ***/
  data test;
    set test;
    by pcode munic numero;
	retain cumul find direct;
	if first.numero then do; find=0; cumul=0; 
                             if last.numero then direct=1; else direct=0;
                         end;
	if find = 0 then do;
	   cumul=cumul+(propmunic/100);
	   if nba < cumul then do; find=1; output; end;
      end;
   run; 

  proc sort data=test;
    by numero;
   run;

  proc sort data=nonapp2;
    by numero;
   run; 

  data appmunic3 fichin;
    merge nonapp2(in=in1) test(in=in2);
	by numero;
	if in1*in2 then output appmunic3;
	else if in1 then output fichin;
   run;

 %end;

/*********************************************************
  * Assignation des indices de dfavorisation  partir ***
  * du code postal seulement                           ***
  * Fichier FCCP+                                      ***
  *******************************************************/
/*** Avec WCF *****/
  data wcf2;
    set wcf;
   run;

     proc sql;
     create table test as
     select fichin.pcode, fichin.numero, nba,
            wcf2.propcp, wcf2.diffmatcp, wcf2.diffsoccp, 
            wcf2.fortmunic, wcf2.fortcp,wcf2.nuniq,
			wcf2.quintmat, wcf2.quintsoc, wcf2.centmat, wcf2.centsoc,
			wcf2.zim, wcf2.zone, wcf2.pr, 
			wcf2.quintmatRC, wcf2.quintsocRC, wcf2.quintmatRMR, wcf2.quintsocRMR, wcf2.quintmatZONE, wcf2.quintsocZONE
     from fichin, wcf2
     where fichin.pcode=wcf2.pcode
	 order by numero,nuniq; 
   quit;

   proc sort data=test;
    by pcode numero;
   run;

   /*** Choix de l'indice de dfavorisation si plus d'un indice
        par code postal ***/
  data test;
    set test;
    by pcode numero;
	retain cumul find direct;
	if first.numero then do; find=0; cumul=0; 
                             if last.numero then direct=1; else direct=0;
                         end;
	if find = 0 then do;
	   cumul=cumul+(propcp/100);
	   if nba < cumul then do; find=1; output; end;
      end;
   run; 

  proc sort data=test;
    by numero;
   run;

  proc sort data=fichin;
    by numero;
   run; 

  data appcp1 nonapp3;
    merge fichin(in=in1) test(in=in2);
	by numero;
	if in1*in2 then output appcp1;
	else if in1 then output nonapp3;
   run;

 /*** Avec PCCFUINIQ *****/
  proc sort data=nonapp3;
    by pcode;
   run;

  proc sort data=pccfuniq;
    by pcode;
   run;

     proc sql;
     create table appcp2 as
     select nonapp3.pcode, nonapp3.numero, nba,
            pccfuniq.propcp, pccfuniq.diffmatcp, pccfuniq.diffsoccp, 
            pccfuniq.fortmunic, pccfuniq.fortcp,pccfuniq.nuniq,
			pccfuniq.quintmat, pccfuniq.quintsoc, pccfuniq.centmat, pccfuniq.centsoc,
			pccfuniq.zim, pccfuniq.zone, pccfuniq.pr, 
			pccfuniq.quintmatRC, pccfuniq.quintsocRC, pccfuniq.quintmatRMR, pccfuniq.quintsocRMR,
			pccfuniq.quintmatZONE, pccfuniq.quintsocZONE
     from nonapp3, pccfuniq
     where nonapp3.pcode=pccfuniq.pcode
	 order by numero,nuniq; 
   quit;


  data nonapp4 ;
    merge nonapp3(in=in2) pccfuniq(in=in1);
	by pcode;
	if in2 and not(in1);
   run;

 /*** Avec PCCFDUPS *****/

  data pccfdup2;
    set pccfdups;
   run;

     proc sql;
     create table test as
     select nonapp4.pcode, nonapp4.numero, nba,
            pccfdup2.propcp, pccfdup2.diffmatcp, pccfdup2.diffsoccp, 
            pccfdup2.fortmunic, pccfdup2.fortcp,pccfdup2.nuniq,
			pccfdup2.quintmat, pccfdup2.quintsoc, pccfdup2.centmat, pccfdup2.centsoc,
			pccfdup2.zim, pccfdup2.zone, pccfdup2.pr, 
			pccfdup2.quintmatRC, pccfdup2.quintsocRC, pccfdup2.quintmatRMR, pccfdup2.quintsocRMR,
			pccfdup2.quintmatZONE, pccfdup2.quintsocZONE
     from nonapp4, pccfdup2
     where nonapp4.pcode=pccfdup2.pcode
	 order by numero,nuniq; 
   quit;

   proc sort data=test;
    by pcode numero;
   run;

   /*** Choix de l'indice de dfavorisation si plus d'un indice
        par code postal ***/
  data test;
    set test;
    by pcode numero;
	retain cumul find direct;
	if first.numero then do; find=0; cumul=0; 
                             if last.numero then direct=1; else direct=0;
                         end;
	if find = 0 then do;
	   cumul=cumul+(propcp/100);
	   if nba < cumul then do; find=1; output; end;
      end;
   run; 

  proc sort data=test;
    by numero;
   run;

  proc sort data=nonapp4;
    by numero;
   run; 

  data appcp3 nonapp5 ;
    merge nonapp4(in=in1) test(in=in2);
	by numero;
	if in1*in2 then output appcp3;
	else if in1 then output nonapp5;
   run;

  data tout;
    set %if &munic ne 0 %then %do; 
                                appmunic1(in=in1) appmunic2(in=in2)
                                appmunic3(in=in3)
                              %end;
	appcp1(in=in4) appcp2(in=in5) appcp3(in=in6)
    nonapp5(in=in7);
	if in7 then direct=0;
	if in5 then do; direct=1;diffmatcp=0; 
                    diffsoccp=0;
				end;
	%if &munic ne 0 %then %do; 
	                           if in1 then base=1;
                               else if in2 then do; direct=1; diffmatmunic=0; 
                                               diffsocmunic=0; base=2;
										   end;
							   else if in3 then base=3;
							   diffmat=diffmatmunic;
	                           diffsoc=diffsocmunic;
                          %end;
	if in4 then base=1;
	else if in5 then base=2;
	else if in6 then base=3;

	if diffmatcp ne . then diffmat=diffmatcp;
 	if diffsoccp ne . then diffsoc=diffsoccp;

	 if in7=1 then do; quintmat = 0; quintsoc = 0; quintmatRC=0; quintsocRC=0; 
                       quintmatRMR=0; quintsocRMR=0;quintmatZONE=0; quintsocZONE=0;  
     end;
	 if quintmat=0 then groupe=6;
	 else if quintmat=. then groupe=5;
	 else if direct=1 then groupe=1;
     else if diffmat=0 and diffsoc=0 then groupe=2;
	 else if fortcp=1 or fortmunic=1 then groupe=3;
     else groupe=4;
     css=zim;
	keep numero base groupe quintmat quintsoc centmat centsoc quintmatRC quintsocRC quintmatRMR quintsocRMR quintmatZONE 
         quintsocZONE zone pr css;
   run;

  proc sort; by numero; run; 

  data fichin;
     set &in;
	 numero=_n_;
    run;

   data tout;
     retain quintmat quintsoc centmat centsoc quintmatRC quintsocRC quintmatRMR quintsocRMR quintmatZONE 
            quintsocZONE zone pr css base groupe numero ;
     set tout;
   run;

  data &out;
    merge fichin tout;
	by numero;
	DATE = '20100114';
	*format groupe fgroupe. base fbase.;
	drop numero;
   run;

proc datasets library=work;
 delete wcf2 pccfdup2 test test1 test2 test3 fichin tout appmunic1 appmunic2 appmunic3 appcp1 
       appcp2 appcp3 nonapp1 nonapp2 nonapp3 nonapp4 nonapp5;
 quit;
%mend assignation2006;


proc format;
  value fgroupe 1="Assignation CP - Indice unique"
  		        2="Assignation non unique mais tous les AD mnent au mme indice"
				3="Assignation non unique mais avec prdominance d'un AD  plus de 90%"
				4="Autre assignation non unique"
				5="Appartient  un AD sans indice"
				6="Aucun appariement CP - AD - Indice";
  value fbase 1="WCF"
			  2="PCFF unique"
			  3="PCCF dups";
 run;

 /*******************************************************************
 * Exemple: %assignation2006(naiss06,cp,cod_mun,naiss06v2);     *********
 * - Le fichier en entre est WORK.NAISS06                  *********
 * - La variable pour le code postal est CP                 *********
 * - La variable pour le code municipal est COD_MUN         *********
 *   (N.B.: on doit mettre 0 (zro) si on ne veut pas       *********
 *   faire l'assignation  l'aide du code municipal         *********
 * - Le fichier rsultant est WORK.NAISS06V2 (identique     *********
 *   au fichier d'entre mais avec les indices de           *********
 *   dfavorisation en quintiles, quartiles et centiles     *********
 *   et les codes de ZONE)                                  *********
 * - NOTE: Les rsultats peuvent varier si l'ordre du       *********
 *   fichier d'entre n'est pas la mme.                    *********
 ********************************************************************/ 

%assignation2006(in=d2006,pcode=CPRESDCD,munic=CMUNRESDCD,out=d2006v2);
